package fr.mael.jiwigo.transverse;

import java.awt.image.BufferedImage;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import javax.imageio.ImageIO;
import fr.mael.jiwigo.om.Image;
import fr.mael.jiwigo.transverse.util.Outil;


/**
   Copyright (c) 2010, Mael
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of jiwigo nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL Mael BE LIABLE FOR ANY
   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   
 * @author mael
 * Management of the real sized pictures
 */
public class ImagesManagement {

	/**
	 * Logger
	 */
	public static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(ImagesManagement.class);
	/**
	 * the index of the current image
	 */
	public static int CURRENT_IMAGE_INDEX;
	/**
	 * the current image
	 */
	public static Image CURRENT_IMAGE;
	/**
	 * images list
	 */
	public static List<Image> LIST_IMAGE;

	/**
	 * cache allows to keep the images in the ram
	 */
	private static HashMap<Integer, BufferedImage> IMAGES_CACHE = new HashMap<Integer, BufferedImage>();

	/**
	 * cache allows to keep the thumbnails in the ram
	 */
	private static HashMap<Integer, BufferedImage> MINIATURE_CACHE = new HashMap<Integer, BufferedImage>();

	/**
	 * If true : the application won't ask the privacy level
	 * to the user each time he adds pictures
	 */
	public static boolean rememberPrivacyLevel = false;

	/**
	 * the privacy level to apply to the pictures
	 */
	public static int privacyLevel = 1;


	/**
	 * gets the current image
	 * @return the image
	 */
	public static Image getCurrentImage() {
		//return LIST_IMAGE.get(CURRENT_IMAGE_INDEX);
		return CURRENT_IMAGE;
	}


	/**
	 * next image
	 */
	public static void next() {
		if (CURRENT_IMAGE_INDEX != LIST_IMAGE.size() - 1) {
			CURRENT_IMAGE_INDEX++;
		} else {
			CURRENT_IMAGE_INDEX = 0;
		}
		CURRENT_IMAGE = LIST_IMAGE.get(CURRENT_IMAGE_INDEX);
	}


	/**
	 * previous image
	 */
	public static void previous() {
		if (CURRENT_IMAGE_INDEX != 0) {
			CURRENT_IMAGE_INDEX--;
		} else {
			CURRENT_IMAGE_INDEX = LIST_IMAGE.size() - 1;
		}
		CURRENT_IMAGE = LIST_IMAGE.get(CURRENT_IMAGE_INDEX);
	}


	/**
	 * 
	 * @param image the current image
	 */
	public static void setCurrentImage(Image image) {
		CURRENT_IMAGE = image;
		int compteur = 0;
		for (Image im : LIST_IMAGE) {
			if (im.equals(image)) {
				CURRENT_IMAGE_INDEX = compteur;
			}
			compteur++;
		}
	}


	/**
	 * Function that allows to load images once
	 * to decrease response delays
	 * @return the image
	 */
	public static BufferedImage getCurrentBufferedImage() {
		if (IMAGES_CACHE.get(CURRENT_IMAGE.getIdentifiant()) == null) {
			try {
				BufferedImage img = ImageIO.read(new URL(CURRENT_IMAGE.getUrl()));
				IMAGES_CACHE.put(CURRENT_IMAGE.getIdentifiant(), img);
			} catch (Exception e) {
				LOG.error(Outil.getStackTrace(e));
			}
		}
		return IMAGES_CACHE.get(CURRENT_IMAGE.getIdentifiant());
	}


	/**
	 * Function that allows to load thimbnails once
	 * to decrease response delays
	 * @return the image
	 */
	public static BufferedImage getMiniatureBufferedImage(Image image) {
		if (MINIATURE_CACHE.get(image.getIdentifiant()) == null) {
			try {
				BufferedImage img = ImageIO.read(new URL(image.getMiniature()));
				MINIATURE_CACHE.put(image.getIdentifiant(), img);
			} catch (Exception e) {
				LOG.error(Outil.getStackTrace(e));
			}
		}

		return MINIATURE_CACHE.get(image.getIdentifiant());
	}
}
