package fr.mael.jiwigo.ui.mainframe;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.HashMap;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import fr.mael.jiwigo.transverse.util.Messages;
import fr.mael.jiwigo.transverse.util.Outil;
import fr.mael.jiwigo.ui.field.HintTextField;
import fr.mael.jiwigo.ui.mainframe.tab.JTabbedPaneWithCloseIcons;


/**
   Copyright (c) 2010, Mael
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of jiwigo nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL Mael BE LIABLE FOR ANY
   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   
 * @author mael
 * Frame principale
 */
public class MainFrame extends JFrame implements ActionListener, KeyListener {

	/**
	 * Logger
	 */
	public static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(MainFrame.class);
	/**
	 * the categories tree
	 */
	private CategoriesTree categoriesTree;
	/**
	 * Splitpane. On the left : the categories tree and on the right the thumbnails
	 * for the current category
	 */
	private JSplitPane splitPane;
	/**
	 * Panel that contains the thumbnail of the current category
	 */
	public static ThumbnailCategoryPanel imagesPanel;
	/**
	 *  Scrollpane that contains the panel above
	 */
	private JScrollPane scrollPaneImagesPanel;
	/**
	 * label that displays messages
	 */
	private JLabel labelMessage;

	/**
	 * menu bar
	 */
	private JMenuBar jMenuBar;

	/**
	 * edition menu
	 */
	private JMenu jMenuEdition;

	/**
	 * preferences menu
	 */
	private JMenuItem jMenuItemPreferences;

	/**
	 * singleton
	 */
	private static MainFrame instance;

	private JProgressBar progressBar;

	private JTabbedPaneWithCloseIcons tabbedPane;

	private HashMap<Integer, Integer> mapsIdPos = new HashMap<Integer, Integer>();

	private HintTextField fieldSearch;


	/**
	 * @return the singleton
	 */
	public static MainFrame getInstance() {
		if (instance == null) {
			instance = new MainFrame();
		}
		return instance;
	}


	/**
	 * private constructor to use a singleton
	 */
	private MainFrame() {
		this.setTitle("Jiwigo v" + Messages.getMessage("version"));
		this.setIconImage(java.awt.Toolkit.getDefaultToolkit().getImage(Outil.getURL("fr/mael/jiwigo/img/icon.png")));
		this.setLayout(new BorderLayout());
		jMenuBar = new JMenuBar();
		labelMessage = new JLabel(Messages.getMessage("welcomeMessage"));
		splitPane = new JSplitPane();
		categoriesTree = new CategoriesTree();
		splitPane.setLeftComponent(categoriesTree);

		tabbedPane = new JTabbedPaneWithCloseIcons();
		splitPane.setRightComponent(tabbedPane);

		this.add(splitPane, BorderLayout.CENTER);
		JPanel panelBas = new JPanel(new BorderLayout());
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		progressBar = new JProgressBar(0, 100);
		panel.add(progressBar);
		panel.add(labelMessage);
		panelBas.add(panel, BorderLayout.WEST);

		fieldSearch = new HintTextField(Messages.getMessage("mainFrame_recherche"));
		fieldSearch.setPreferredSize(new Dimension(150, 25));
		fieldSearch.addKeyListener(this);
		panelBas.add(fieldSearch, BorderLayout.EAST);

		jMenuEdition = new JMenu(Messages.getMessage("mainFrame_editionMenu"));
		jMenuBar.add(jMenuEdition);
		jMenuItemPreferences = new JMenuItem(Messages.getMessage("mainFrame_preferencesMenu"));
		jMenuItemPreferences.addActionListener(this);
		jMenuEdition.add(jMenuItemPreferences);

		this.setJMenuBar(jMenuBar);
		this.add(panelBas, BorderLayout.SOUTH);
		this.setSize(900, 600);
		this.setLocationRelativeTo(null);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setResizable(true);
	}


	public void addTabb(IThumbnailPanel panel) {
		JScrollPane scrollPaneImagesPanel = new JScrollPane((JPanel) panel, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPaneImagesPanel.setPreferredSize(new Dimension(900, 600));
		boolean found = false;
		boolean isSearch = false;
		if (panel instanceof ThumbnailSearchPanel) {
			isSearch = true;
		}

		for (int i = 0; i < tabbedPane.getTabCount(); i++) {
			JScrollPane scroll = (JScrollPane) tabbedPane.getComponentAt(i);
			IThumbnailPanel thumbPan = (IThumbnailPanel) scroll.getViewport().getComponents()[0];
			if (thumbPan.getCategory().getIdentifiant().equals(panel.getCategory().getIdentifiant())) {
				//only if it's not for a re
				if (!(panel instanceof ThumbnailSearchPanel)) {
					tabbedPane.setSelectedIndex(i);
					found = true;
					break;
				}
			}
		}
		//if it's not for a research, the title of the tab
		//is the name of the category
		if (!found && !isSearch) {
			tabbedPane.addTab(panel.getCategory().getNom(), scrollPaneImagesPanel,
					new ImageIcon(Outil.getURL("fr/mael/jiwigo/img/closetab.png")));
			//if it's for a research, the title of the tab
			//if the query string
		} else if (!found && isSearch) {
			String queryString = ((ThumbnailSearchPanel) panel).getQueryString();
			tabbedPane.addTab(Messages.getMessage("mainFrame_search") + queryString, scrollPaneImagesPanel,
					new ImageIcon(Outil.getURL("fr/mael/jiwigo/img/closetab.png")));
		}

	}


	/**
	 * displays the frame
	 */
	public void showUi() {
		this.setVisible(true);
	}


	/**
	 * Affichage d'un message de réussite
	 * @param message
	 */
	public void setMessage(String message) {
		this.labelMessage.setText(message);
	}


	@Override
	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getSource().equals(jMenuItemPreferences)) {
			new PreferencesDialog(this);
		}

	}


	public static void reduceSizeOfComponent(JComponent comp) {
		comp.setMaximumSize(comp.getPreferredSize());
	}


	/**
	 * @return the progressBar
	 */
	public JProgressBar getProgressBar() {
		return progressBar;
	}


	/**
	 * @return the mapsIdPos
	 */
	public HashMap<Integer, Integer> getMapsIdPos() {
		return mapsIdPos;
	}


	@Override
	public void keyPressed(KeyEvent paramKeyEvent) {
		if (paramKeyEvent.getKeyCode() == KeyEvent.VK_ENTER) {
			String queryString = fieldSearch.getText();
			ThumbnailSearchPanel searchPanel = new ThumbnailSearchPanel(queryString);
			addTabb(searchPanel);
		}
	}


	@Override
	public void keyReleased(KeyEvent paramKeyEvent) {
	}


	@Override
	public void keyTyped(KeyEvent paramKeyEvent) {
	}

}
